@@ -18,15 +18,16 @@ from typing import (
     cast,
 )
 
-from pydantic.utils import lenient_issubclass
+import pymongo
 from pymongo import MongoClient
 from pymongo.client_session import ClientSession
 from pymongo.collection import Collection
 from pymongo.command_cursor import CommandCursor
 from pymongo.database import Database
 
-from odmantic.exceptions import DocumentNotFoundError
+from odmantic.exceptions import DocumentNotFoundError, DuplicateKeyError
 from odmantic.field import FieldProxy, ODMReference
+from odmantic.index import ODMBaseIndex
 from odmantic.model import Model
 from odmantic.query import QueryExpression, SortExpression, and_
 from odmantic.session import (
@@ -37,6 +38,7 @@ from odmantic.session import (
     SyncSessionBase,
     SyncTransaction,
 )
+from odmantic.typing import lenient_issubclass
 
 try:
     import motor
@@ -343,6 +345,51 @@ class AIOEngine(BaseEngine):
         assert not isinstance(session, AIOSessionBase)  # Abstract class
         return session
 
+    async def configure_database(
+        self,
+        models: Sequence[Type[ModelType]],
+        *,
+        update_existing_indexes: bool = False,
+        session: SyncSessionType = None,
+    ) -> None:
+        """Apply model constraints to the database.
+
+        Args:
+            models: list of models to initialize the database with
+            update_existing_indexes: conflicting indexes will be dropped before creation
+            session: an optional session to use for the operation
+
+        <!---
+        #noqa: DAR401 pymongo.errors.OperationFailure
+        -->
+        """
+        driver_session = self._get_session(session)
+        for model in models:
+            collection = self.get_collection(model)
+            for index in model.__indexes__():
+                pymongo_index = (
+                    index.get_pymongo_index()
+                    if isinstance(index, ODMBaseIndex)
+                    else index
+                )
+                try:
+                    await collection.create_indexes(
+                        [pymongo_index], session=driver_session
+                    )
+                except pymongo.errors.OperationFailure as exc:
+                    if update_existing_indexes and getattr(exc, "code", None) in (
+                        85,  # aka IndexOptionsConflict
+                        86,  # aka IndexKeySpecsConflict for MongoDB > 5
+                    ):
+                        await collection.drop_index(
+                            pymongo_index.document["name"], session=driver_session
+                        )
+                        await collection.create_indexes(
+                            [pymongo_index], session=driver_session
+                        )
+                    else:
+                        raise
+
     def session(self) -> AIOSession:
         """Get a new session for the engine to allow ordering sequential operations.
 
@@ -472,18 +519,19 @@ class AIOEngine(BaseEngine):
             sub_instance = cast(Model, getattr(instance, ref_field_name))
             await self._save(sub_instance, session)
 
-        fields_to_update = (
-            instance.__fields_modified__ | instance.__mutable_fields__
-        ) - set([instance.__primary_field__])
+        fields_to_update = instance.__fields_modified__ | instance.__mutable_fields__
         if len(fields_to_update) > 0:
             doc = instance.doc(include=fields_to_update)
             collection = self.get_collection(type(instance))
-            await collection.update_one(
-                {"_id": getattr(instance, instance.__primary_field__)},
-                {"$set": doc},
-                upsert=True,
-                session=session,
-            )
+            try:
+                await collection.update_one(
+                    instance.doc(include={instance.__primary_field__}),
+                    {"$set": doc},
+                    upsert=True,
+                    session=session,
+                )
+            except pymongo.errors.DuplicateKeyError as e:
+                raise DuplicateKeyError(instance, e)
             object.__setattr__(instance, "__fields_modified__", set())
         return instance
 
@@ -509,12 +557,16 @@ class AIOEngine(BaseEngine):
         Returns:
             the saved instance
 
+        Raises:
+            DuplicateKeyError: the instance is duplicated according to a unique index.
+
         NOTE:
             The save operation actually modify the instance argument in place. However,
             the instance is still returned for convenience.
 
         <!---
         #noqa: DAR401 TypeError
+        #noqa: DAR402 DuplicateKeyError
         -->
         """
         if not isinstance(instance, Model):
@@ -548,9 +600,16 @@ class AIOEngine(BaseEngine):
         Returns:
             the saved instances
 
+        Raises:
+            DuplicateKeyError: an instance is duplicated according to a unique index.
+
         NOTE:
             The save_all operation actually modify the arguments in place. However, the
             instances are still returned for convenience.
+
+        <!---
+        #noqa: DAR402 DuplicateKeyError
+        -->
         """
         if session:
             added_instances = [
@@ -698,6 +757,49 @@ class SyncEngine(BaseEngine):
         assert not isinstance(session, SyncSessionBase)  # Abstract class
         return session
 
+    def configure_database(
+        self,
+        models: Sequence[Type[ModelType]],
+        *,
+        update_existing_indexes: bool = False,
+        session: SyncSessionType = None,
+    ) -> None:
+        """Apply model constraints to the database.
+
+        Args:
+            models: list of models to initialize the database with
+            update_existing_indexes: conflicting indexes will be dropped before creation
+            session: an optional session to use for the operation
+
+        <!---
+        #noqa: DAR401 pymongo.errors.OperationFailure
+        -->
+        """
+        driver_session = self._get_session(session)
+        for model in models:
+            collection = self.get_collection(model)
+            for index in model.__indexes__():
+                pymongo_index = (
+                    index.get_pymongo_index()
+                    if isinstance(index, ODMBaseIndex)
+                    else index
+                )
+                try:
+                    collection.create_indexes([pymongo_index], session=driver_session)
+                except pymongo.errors.OperationFailure as exc:
+                    if update_existing_indexes and getattr(exc, "code", None) in (
+                        85,  # aka IndexOptionsConflict
+                        86,  # aka IndexKeySpecsConflict for MongoDB > 5
+                    ):
+                        collection.drop_index(
+                            pymongo_index.document["name"], session=driver_session
+                        )
+                        collection.create_indexes(
+                            [pymongo_index], session=driver_session
+                        )
+                    else:
+                        raise
+
     def session(self) -> SyncSession:
         """Get a new session for the engine to allow ordering sequential operations.
 
@@ -823,18 +925,19 @@ class SyncEngine(BaseEngine):
             sub_instance = cast(Model, getattr(instance, ref_field_name))
             self._save(sub_instance, session)
 
-        fields_to_update = (
-            instance.__fields_modified__ | instance.__mutable_fields__
-        ) - set([instance.__primary_field__])
+        fields_to_update = instance.__fields_modified__ | instance.__mutable_fields__
         if len(fields_to_update) > 0:
             doc = instance.doc(include=fields_to_update)
             collection = self.get_collection(type(instance))
-            collection.update_one(
-                {"_id": getattr(instance, instance.__primary_field__)},
-                {"$set": doc},
-                upsert=True,
-                session=session,
-            )
+            try:
+                collection.update_one(
+                    instance.doc(include={instance.__primary_field__}),
+                    {"$set": doc},
+                    upsert=True,
+                    session=session,
+                )
+            except pymongo.errors.DuplicateKeyError as e:
+                raise DuplicateKeyError(instance, e)
             object.__setattr__(instance, "__fields_modified__", set())
         return instance
 
@@ -860,12 +963,16 @@ class SyncEngine(BaseEngine):
         Returns:
             the saved instance
 
+        Raises:
+            DuplicateKeyError: the instance is duplicated according to a unique index.
+
         NOTE:
             The save operation actually modify the instance argument in place. However,
             the instance is still returned for convenience.
 
         <!---
         #noqa: DAR401 TypeError
+        #noqa: DAR402 DuplicateKeyError
         -->
         """
         if not isinstance(instance, Model):
@@ -900,9 +1007,15 @@ class SyncEngine(BaseEngine):
         Returns:
             the saved instances
 
+        Raises:
+            DuplicateKeyError: an instance is duplicated according to a unique index.
+
         NOTE:
             The save_all operation actually modify the arguments in place. However, the
             instances are still returned for convenience.
+        <!---
+        #noqa: DAR402 DuplicateKeyError
+        -->
         """
         if session:
             added_instances = [
