@@ -28,6 +28,7 @@ from typing import (
 
 import bson
 import pydantic
+import pymongo
 from pydantic.error_wrappers import ErrorWrapper, ValidationError
 from pydantic.fields import Field as PDField
 from pydantic.fields import FieldInfo as PDFieldInfo
@@ -35,8 +36,6 @@ from pydantic.fields import Undefined
 from pydantic.main import BaseModel
 from pydantic.tools import parse_obj_as
 from pydantic.typing import is_classvar, resolve_annotations
-from pydantic.utils import lenient_issubclass
-from typing_extensions import dataclass_transform
 
 from odmantic.bson import (
     _BSON_SUBSTITUTED_FIELDS,
@@ -48,6 +47,7 @@ from odmantic.config import BaseODMConfig, validate_config
 from odmantic.exceptions import (
     DocumentParsingError,
     ErrorList,
+    IncorrectGenericEmbeddedModelValue,
     KeyNotFoundInDocumentError,
     ReferencedDocumentNotFoundError,
 )
@@ -55,12 +55,25 @@ from odmantic.field import (
     Field,
     FieldProxy,
     ODMBaseField,
+    ODMBaseIndexableField,
     ODMEmbedded,
+    ODMEmbeddedGeneric,
     ODMField,
     ODMFieldInfo,
     ODMReference,
 )
+from odmantic.index import Index, ODMBaseIndex, ODMSingleFieldIndex
 from odmantic.reference import ODMReferenceInfo
+from odmantic.typing import (
+    GenericAlias,
+    Literal,
+    dataclass_transform,
+    get_args,
+    get_first_type_argument_subclassing,
+    get_origin,
+    is_type_argument_subclass,
+    lenient_issubclass,
+)
 from odmantic.utils import (
     is_dunder,
     raise_on_invalid_collection_name,
@@ -167,17 +180,11 @@ def validate_type(type_: Type) -> Type:
     if subst_type is not None:
         return subst_type
 
-    # Typing replacement
-    # 3.7+:
-    # https://github.com/python/cpython/blob/e022bbc169ca1428dc3017187012de17ce6e0bc7/Lib/typing.py#L605
-    # 3.6:
-    # https://github.com/python/cpython/blob/aed26482c7baab078f39d5cd52216fb8ee9f276f/Lib/typing.py#L570
-    type_origin: Optional[Type] = getattr(type_, "__origin__", None)
-    if type_origin is not None:
-        type_args: Tuple[Type, ...] = getattr(type_, "__args__", ())
+    type_origin: Optional[Type] = get_origin(type_)
+    if type_origin is not None and type_origin is not Literal:
+        type_args: Tuple[Type, ...] = get_args(type_)
         new_arg_types = tuple(validate_type(subtype) for subtype in type_args)
-        setattr(type_, "__args__", new_arg_types)
-
+        type_ = GenericAlias(type_origin, new_arg_types)
     return type_
 
 
@@ -237,16 +244,58 @@ class BaseModelMetaclass(pydantic.main.ModelMetaclass):
                         value.key_name if value.key_name is not None else field_name
                     )
                     primary_field = value.primary_field
+                    index = value.index
+                    unique = value.unique
                 else:
                     key_name = field_name
                     primary_field = False
+                    index = False
+                    unique = False
 
                 odm_fields[field_name] = ODMEmbedded(
                     primary_field=primary_field,
                     model=field_type,
                     key_name=key_name,
                     model_config=config,
+                    index=index,
+                    unique=unique,
                 )
+
+            elif is_type_argument_subclass(field_type, EmbeddedModel):
+                if isinstance(value, ODMFieldInfo):
+                    if value.primary_field:
+                        raise TypeError(
+                            "Declaring a generic type of embedded models as a primary "
+                            f"field is not allowed: {field_name} in {name}"
+                        )
+                    namespace[field_name] = value.pydantic_field_info
+                    key_name = (
+                        value.key_name if value.key_name is not None else field_name
+                    )
+                    index = value.index
+                    unique = value.unique
+                else:
+                    key_name = field_name
+                    index = False
+                    unique = False
+                model = get_first_type_argument_subclassing(field_type, EmbeddedModel)
+                assert model is not None
+                if len(model.__references__) > 0:
+                    raise TypeError(
+                        "Declaring a generic type of embedded models containing "
+                        f"references is not allowed: {field_name} in {name}"
+                    )
+                generic_origin = get_origin(field_type)
+                assert generic_origin is not None
+                odm_fields[field_name] = ODMEmbeddedGeneric(
+                    model=model,
+                    generic_origin=generic_origin,
+                    key_name=key_name,
+                    model_config=config,
+                    index=index,
+                    unique=unique,
+                )
+
             elif lenient_issubclass(field_type, Model):
                 if not isinstance(value, ODMReferenceInfo):
                     raise TypeError(
@@ -270,6 +319,8 @@ class BaseModelMetaclass(pydantic.main.ModelMetaclass):
                         primary_field=value.primary_field,
                         key_name=key_name,
                         model_config=config,
+                        index=value.index,
+                        unique=value.unique,
                     )
                     namespace[field_name] = value.pydantic_field_info
 
@@ -290,11 +341,12 @@ class BaseModelMetaclass(pydantic.main.ModelMetaclass):
                         primary_field=False, key_name=field_name, model_config=config
                     )
 
+        # NOTE: Duplicate key detection make sur that at most one primary key is
+        # defined
         duplicate_key = find_duplicate_key(odm_fields.values())
         if duplicate_key is not None:
             raise TypeError(f"Duplicated key_name: {duplicate_key} in {name}")
-        # NOTE: Duplicate key detection make sur that at most one primary key is
-        # defined
+
         namespace["__annotations__"] = annotations
         namespace["__odm_fields__"] = odm_fields
         namespace["__references__"] = tuple(references)
@@ -463,7 +515,9 @@ class _BaseODMModel(pydantic.BaseModel, metaclass=ABCMeta):
         __mutable_fields__: ClassVar[FrozenSet[str]] = frozenset()
         __references__: ClassVar[Tuple[str, ...]] = ()
         __pydantic_model__: ClassVar[Type[BaseBSONModel]]
-        __fields_modified__: Set[str] = set()
+        # __fields_modified__ is not a ClassVar but this allows to hide this field from
+        # the dataclass transform generated constructor
+        __fields_modified__: ClassVar[Set[str]] = set()
 
     __slots__ = ("__fields_modified__",)
 
@@ -651,12 +705,30 @@ class _BaseODMModel(pydantic.BaseModel, metaclass=ABCMeta):
                 doc[field.key_name] = raw_doc[field_name][field.model.__primary_field__]
             elif isinstance(field, ODMEmbedded):
                 doc[field.key_name] = self.__doc(raw_doc[field_name], field.model, None)
+            elif isinstance(field, ODMEmbeddedGeneric):
+                if field.generic_origin is dict:
+                    doc[field.key_name] = {
+                        item_key: self.__doc(item_value, field.model)
+                        for item_key, item_value in raw_doc[field_name].items()
+                    }
+                else:
+                    doc[field.key_name] = [
+                        self.__doc(item, field.model) for item in raw_doc[field_name]
+                    ]
             elif field_name in model.__bson_serialized_fields__:
                 doc[field.key_name] = model.__fields__[field_name].type_.__bson__(
                     raw_doc[field_name]
                 )
             else:
                 doc[field.key_name] = raw_doc[field_name]
+
+        if model.Config.extra == "allow":
+            extras = set(raw_doc.keys()) - set(model.__odm_fields__.keys())
+            for extra in extras:
+                value = raw_doc[extra]
+                subst_type = validate_type(type(value))
+                bson_serialization_method = getattr(subst_type, "__bson__", lambda x: x)
+                doc[extra] = bson_serialization_method(raw_doc[extra])
         return doc
 
     def doc(self, include: Optional["AbstractSetIntStr"] = None) -> Dict[str, Any]:
@@ -705,7 +777,7 @@ class _BaseODMModel(pydantic.BaseModel, metaclass=ABCMeta):
         return instance
 
     @classmethod
-    def _parse_doc_to_obj(
+    def _parse_doc_to_obj(  # noqa C901 # TODO: refactor document parsing
         cls: Type[BaseT], raw_doc: Dict, base_loc: Tuple[str, ...] = ()
     ) -> Tuple[ErrorList, Dict[str, Any]]:
         errors: ErrorList = []
@@ -726,8 +798,76 @@ class _BaseODMModel(pydantic.BaseModel, metaclass=ABCMeta):
                     )
                     errors.extend(sub_errors)
                     obj[field_name] = sub_obj
+            elif isinstance(field, ODMEmbedded):
+                value = raw_doc.get(field.key_name, Undefined)
+                if value is not Undefined:
+                    sub_errors, value = field.model._parse_doc_to_obj(
+                        value, base_loc=base_loc + (field_name,)
+                    )
+                    errors.extend(sub_errors)
+                else:
+                    if not field.is_required_in_doc():
+                        value = field.get_default_importing_value()
+                    if value is Undefined:
+                        errors.append(
+                            ErrorWrapper(
+                                exc=KeyNotFoundInDocumentError(field.key_name),
+                                loc=base_loc + (field_name,),
+                            )
+                        )
+                obj[field_name] = value
+            elif isinstance(field, ODMEmbeddedGeneric):
+                value = Undefined
+                raw_value = raw_doc.get(field.key_name, Undefined)
+                if raw_value is not Undefined:
+                    if isinstance(raw_value, list) and (
+                        field.generic_origin is list
+                        or field.generic_origin is tuple
+                        or field.generic_origin is set
+                    ):
+                        value = []
+                        for i, item in enumerate(raw_value):
+                            sub_errors, item = field.model._parse_doc_to_obj(
+                                item, base_loc=base_loc + (field_name, f"[{i}]")
+                            )
+                            if len(sub_errors) > 0:
+                                errors.extend(sub_errors)
+                            else:
+                                value.append(item)
+                        obj[field_name] = value
+                    elif isinstance(raw_value, dict) and field.generic_origin is dict:
+                        value = {}
+                        for item_key, item_value in raw_value.items():
+                            sub_errors, item_value = field.model._parse_doc_to_obj(
+                                item_value,
+                                base_loc=base_loc + (field_name, f'["{item_key}"]'),
+                            )
+                            if len(sub_errors) > 0:
+                                errors.extend(sub_errors)
+                            else:
+                                value[item_key] = item_value
+                        obj[field_name] = value
+                    else:
+                        errors.append(
+                            ErrorWrapper(
+                                exc=IncorrectGenericEmbeddedModelValue(raw_value),
+                                loc=base_loc + (field_name,),
+                            )
+                        )
+                else:
+                    if not field.is_required_in_doc():
+                        value = field.get_default_importing_value()
+                    if value is Undefined:
+                        errors.append(
+                            ErrorWrapper(
+                                exc=KeyNotFoundInDocumentError(field.key_name),
+                                loc=base_loc + (field_name,),
+                            )
+                        )
+                    else:
+                        obj[field_name] = value
             else:
-                field = cast(Union[ODMField, ODMEmbedded], field)
+                field = cast(ODMField, field)
                 value = raw_doc.get(field.key_name, Undefined)
                 if value is Undefined and not field.is_required_in_doc():
                     value = field.get_default_importing_value()
@@ -742,6 +882,11 @@ class _BaseODMModel(pydantic.BaseModel, metaclass=ABCMeta):
                 else:
                     obj[field_name] = value
 
+        if cls.Config.extra == "allow":
+            extras = set(raw_doc.keys()) - set(obj.keys())
+            for extra in extras:
+                obj[extra] = raw_doc[extra]
+
         return errors, obj
 
 
@@ -756,7 +901,7 @@ class Model(_BaseODMModel, metaclass=ModelMetaclass):
         __collection__: ClassVar[str] = ""
         __primary_field__: ClassVar[str] = ""
 
-        id: Union[ObjectId, Any] = None  # TODO fix basic id field typing
+        id: Union[ObjectId, Any] = Field(init=False)  # TODO fix basic id field typing
 
     def __setattr__(self, name: str, value: Any) -> None:
         if name == self.__primary_field__:
@@ -766,6 +911,24 @@ class Model(_BaseODMModel, metaclass=ModelMetaclass):
             )
         super().__setattr__(name, value)
 
+    @classmethod
+    def __indexes__(cls) -> Tuple[Union[ODMBaseIndex, pymongo.IndexModel], ...]:
+        indexes: List[Union[ODMBaseIndex, pymongo.IndexModel]] = []
+        for field in cls.__odm_fields__.values():
+            if isinstance(field, ODMBaseIndexableField) and (
+                field.index or field.unique
+            ):
+                indexes.append(
+                    ODMSingleFieldIndex(
+                        key_name=field.key_name,
+                        unique=field.unique,
+                    )
+                )
+
+        for index in cast(BaseODMConfig, cls.Config).indexes():
+            indexes.append(index.to_odm_index() if isinstance(index, Index) else index)
+        return tuple(indexes)
+
     def update(
         self,
         patch_object: Union[BaseModel, Dict[str, Any]],
